#include "stdafx.h"
#include "cunicode.h"
#include "stdlib.h"

char usysychecked=0;

BOOL usys()
{
	if (!usysychecked) {
		OSVERSIONINFO vx;
		vx.dwOSVersionInfoSize=sizeof(vx);
		GetVersionEx(&vx);
		if (vx.dwPlatformId==VER_PLATFORM_WIN32_NT)
			usysychecked=1;
		else
			usysychecked=2;
	}
	return (usysychecked==1);
}

char* walcopy(char* outname, const wchar_t* inname, int maxlen)
{
	if (inname) {
		WideCharToMultiByte(CP_ACP, 0, inname, -1, outname, maxlen, NULL, NULL);
		outname[maxlen]=0;
		return outname;
	} else
		return NULL;
}

wchar_t* awlcopy(wchar_t* outname, const char* inname, int maxlen)
{
	if (inname) {
		MultiByteToWideChar(CP_ACP, 0, inname, -1, outname, maxlen);
		outname[maxlen]=0;
		return outname;
	} else
		return NULL;
}

wchar_t* wcslcpy(wchar_t *str1, const wchar_t *str2, int imaxlen)
{
	if ((int)wcslen(str2)>=imaxlen-1) {
		wcsncpy(str1, str2, imaxlen-1);
		str1[imaxlen-1]=0;
	} else
		wcscpy(str1, str2);
	return str1;
}

wchar_t* wcslcat(wchar_t *str1, const wchar_t *str2, int imaxlen)
{
	int l1=(int)wcslen(str1);
	if ((int)wcslen(str2)+l1>=imaxlen-1) {
		wcsncpy(str1+l1, str2, imaxlen-1-l1);
		str1[imaxlen-1]=0;
	} else
		wcscat(str1, str2);
	return str1;
}

// return true if name wasn't cut
BOOL MakeExtraLongNameW(wchar_t* outbuf, const wchar_t* inbuf, int maxlen)
{
	if (wcslen(inbuf)>259) {
		if (inbuf[0]=='\\' && inbuf[1]=='\\') {   // UNC-Path! Use \\?\UNC\server\share\subdir\name.ext
			wcslcpy(outbuf, L"\\\\?\\UNC", maxlen);
			wcslcat(outbuf, inbuf+1, maxlen);
		} else {
			wcslcpy(outbuf, L"\\\\?\\", maxlen);
			wcslcat(outbuf, inbuf, maxlen);
		}
	} else
		wcslcpy(outbuf, inbuf, maxlen);
	return (int)wcslen(inbuf)+4<=maxlen;
}

/***********************************************************************************************/

void copyfinddatawa(WIN32_FIND_DATA *lpFindFileDataA, WIN32_FIND_DATAW *lpFindFileDataW)
{
	walcopy(lpFindFileDataA->cAlternateFileName, lpFindFileDataW->cAlternateFileName, sizeof(lpFindFileDataW->cAlternateFileName)-1);
	walcopy(lpFindFileDataA->cFileName, lpFindFileDataW->cFileName, sizeof(lpFindFileDataW->cFileName)-1);
	lpFindFileDataA->dwFileAttributes=lpFindFileDataW->dwFileAttributes;
	lpFindFileDataA->dwReserved0=lpFindFileDataW->dwReserved0;
	lpFindFileDataA->dwReserved1=lpFindFileDataW->dwReserved1;
	lpFindFileDataA->ftCreationTime=lpFindFileDataW->ftCreationTime;
	lpFindFileDataA->ftLastAccessTime=lpFindFileDataW->ftLastAccessTime;
	lpFindFileDataA->ftLastWriteTime=lpFindFileDataW->ftLastWriteTime;
	lpFindFileDataA->nFileSizeHigh=lpFindFileDataW->nFileSizeHigh;
	lpFindFileDataA->nFileSizeLow=lpFindFileDataW->nFileSizeLow;
}

void copyfinddataaw(WIN32_FIND_DATAW *lpFindFileDataW, WIN32_FIND_DATA *lpFindFileDataA)
{
	awlcopy(lpFindFileDataW->cAlternateFileName, lpFindFileDataA->cAlternateFileName, countof(lpFindFileDataW->cAlternateFileName)-1);
	awlcopy(lpFindFileDataW->cFileName, lpFindFileDataA->cFileName, countof(lpFindFileDataW->cFileName)-1);
	lpFindFileDataW->dwFileAttributes=lpFindFileDataA->dwFileAttributes;
	lpFindFileDataW->dwReserved0=lpFindFileDataA->dwReserved0;
	lpFindFileDataW->dwReserved1=lpFindFileDataA->dwReserved1;
	lpFindFileDataW->ftCreationTime=lpFindFileDataA->ftCreationTime;
	lpFindFileDataW->ftLastAccessTime=lpFindFileDataA->ftLastAccessTime;
	lpFindFileDataW->ftLastWriteTime=lpFindFileDataA->ftLastWriteTime;
	lpFindFileDataW->nFileSizeHigh=lpFindFileDataA->nFileSizeHigh;
	lpFindFileDataW->nFileSizeLow=lpFindFileDataA->nFileSizeLow;
}

/***********************************************************************************************/

BOOL CopyFileT(wchar_t* lpExistingFileName, wchar_t* lpNewFileName, BOOL bFailIfExists)
{
	if (usys()) {
		wchar_t wbuf1[wdirtypemax+longnameprefixmax], wbuf2[wdirtypemax+longnameprefixmax];
		if (MakeExtraLongNameW(wbuf1, lpExistingFileName, wdirtypemax-1+longnameprefixmax) &&
			MakeExtraLongNameW(wbuf2, lpNewFileName, wdirtypemax-1+longnameprefixmax))
			return CopyFileW(wbuf1, wbuf2, bFailIfExists);
		else
			return false;
	} else {
		char buf1[_MAX_PATH], buf2[_MAX_PATH];
		return CopyFile(wafilenamecopy(buf1, lpExistingFileName), wafilenamecopy(buf2, lpNewFileName), bFailIfExists);
	}
}

BOOL CreateDirectoryT(wchar_t* lpPathName, LPSECURITY_ATTRIBUTES lpSecurityAttributes)
{
	if (usys()) {
		wchar_t wbuf[wdirtypemax+longnameprefixmax];
		if (MakeExtraLongNameW(wbuf, lpPathName, wdirtypemax-1+longnameprefixmax))
			return CreateDirectoryW(wbuf, lpSecurityAttributes);
		else
			return false;
	} else {
		char buf[_MAX_PATH];
		return CreateDirectory(wafilenamecopy(buf, lpPathName), lpSecurityAttributes);
	}
}

BOOL RemoveDirectoryT(wchar_t* lpPathName)
{
	if (usys()) {
		wchar_t wbuf[wdirtypemax+longnameprefixmax];
		if (MakeExtraLongNameW(wbuf, lpPathName, wdirtypemax-1+longnameprefixmax))
			return RemoveDirectoryW(wbuf);
		else
			return false;
	} else {
		char buf[_MAX_PATH];
		return RemoveDirectory(wafilenamecopy(buf, lpPathName));
	}
}

BOOL DeleteFileT(wchar_t* lpFileName)
{
	if (usys()) {
		wchar_t wbuf[wdirtypemax+longnameprefixmax];
		if (MakeExtraLongNameW(wbuf, lpFileName, wdirtypemax-1+longnameprefixmax))
			return DeleteFileW(wbuf);
		else
			return false;
	} else {
		char buf[_MAX_PATH];
		return DeleteFile(wafilenamecopy(buf, lpFileName));
	}
}

BOOL MoveFileT(wchar_t* lpExistingFileName, wchar_t* lpNewFileName)
{
	if (usys()) {
		wchar_t wbuf1[wdirtypemax+longnameprefixmax], wbuf2[wdirtypemax+longnameprefixmax];
		if (MakeExtraLongNameW(wbuf1, lpExistingFileName, wdirtypemax-1+longnameprefixmax) &&
			MakeExtraLongNameW(wbuf2, lpNewFileName, wdirtypemax-1+longnameprefixmax))
			return MoveFileW(wbuf1, wbuf2);
		else
			return false;
	} else {
		char buf1[_MAX_PATH], buf2[_MAX_PATH];
		return MoveFile(wafilenamecopy(buf1, lpExistingFileName), wafilenamecopy(buf2, lpNewFileName));
	}
}

BOOL SetFileAttributesT(wchar_t* lpFileName, DWORD dwFileAttributes)
{
	if (usys()) {
		wchar_t wbuf[wdirtypemax+longnameprefixmax];
		if (MakeExtraLongNameW(wbuf, lpFileName, wdirtypemax-1+longnameprefixmax))
			return SetFileAttributesW(wbuf, dwFileAttributes);
		else
			return false;
	} else {
		char buf[_MAX_PATH];
		return SetFileAttributes(wafilenamecopy(buf, lpFileName), dwFileAttributes);
	}
}

HANDLE CreateFileT(const wchar_t* lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode, 
  LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreationDisposition, 
  DWORD dwFlagsAndAttributes, HANDLE hTemplateFile)
{
	if (usys()) {
		wchar_t wbuf[wdirtypemax+longnameprefixmax];
		if (MakeExtraLongNameW(wbuf, lpFileName, wdirtypemax-1+longnameprefixmax))
			return CreateFileW(wbuf, dwDesiredAccess, dwShareMode, 
				lpSecurityAttributes, dwCreationDisposition, 
				dwFlagsAndAttributes, hTemplateFile);
		else
			return INVALID_HANDLE_VALUE;
	} else {
		char buf[_MAX_PATH];
		return CreateFile(wafilenamecopy(buf, lpFileName), dwDesiredAccess, dwShareMode, 
			lpSecurityAttributes, dwCreationDisposition, 
			dwFlagsAndAttributes, hTemplateFile);
	}
}

HANDLE FindFirstFileT(const wchar_t* lpFileName, LPWIN32_FIND_DATAW lpFindFileData)
{
	if (usys()) {
		wchar_t wbuf[wdirtypemax+longnameprefixmax];
		if (MakeExtraLongNameW(wbuf, lpFileName, wdirtypemax-1+longnameprefixmax))
			return FindFirstFileW(wbuf, lpFindFileData);
		else
			return INVALID_HANDLE_VALUE;
	} else {
		char buf[_MAX_PATH];
		WIN32_FIND_DATA FindFileDataA;
		HANDLE retval=FindFirstFile(wafilenamecopy(buf, lpFileName), &FindFileDataA);
		if (retval!=INVALID_HANDLE_VALUE) {
			awlcopy(lpFindFileData->cAlternateFileName, FindFileDataA.cAlternateFileName, countof(lpFindFileData->cAlternateFileName)-1);
			awlcopy(lpFindFileData->cFileName, FindFileDataA.cFileName, countof(lpFindFileData->cFileName)-1);
			lpFindFileData->dwFileAttributes=FindFileDataA.dwFileAttributes;
			lpFindFileData->dwReserved0=FindFileDataA.dwReserved0;
			lpFindFileData->dwReserved1=FindFileDataA.dwReserved1;
			lpFindFileData->ftCreationTime=FindFileDataA.ftCreationTime;
			lpFindFileData->ftLastAccessTime=FindFileDataA.ftLastAccessTime;
			lpFindFileData->ftLastWriteTime=FindFileDataA.ftLastWriteTime;
			lpFindFileData->nFileSizeHigh=FindFileDataA.nFileSizeHigh;
			lpFindFileData->nFileSizeLow=FindFileDataA.nFileSizeLow;
		}
		return retval;
	}
}

BOOL FindNextFileT(HANDLE hFindFile, LPWIN32_FIND_DATAW lpFindFileData)
{
	if (usys()) {
		return FindNextFileW(hFindFile, lpFindFileData);
	} else {
		WIN32_FIND_DATA FindFileDataA;
		memset(&FindFileDataA, 0, sizeof(FindFileDataA));
		BOOL retval=FindNextFile(hFindFile, &FindFileDataA);
		if (retval) {
			awlcopy(lpFindFileData->cAlternateFileName, FindFileDataA.cAlternateFileName, countof(lpFindFileData->cAlternateFileName)-1);
			awlcopy(lpFindFileData->cFileName, FindFileDataA.cFileName, countof(lpFindFileData->cFileName)-1);
			lpFindFileData->dwFileAttributes=FindFileDataA.dwFileAttributes;
			lpFindFileData->dwReserved0=FindFileDataA.dwReserved0;
			lpFindFileData->dwReserved1=FindFileDataA.dwReserved1;
			lpFindFileData->ftCreationTime=FindFileDataA.ftCreationTime;
			lpFindFileData->ftLastAccessTime=FindFileDataA.ftLastAccessTime;
			lpFindFileData->ftLastWriteTime=FindFileDataA.ftLastWriteTime;
			lpFindFileData->nFileSizeHigh=FindFileDataA.nFileSizeHigh;
			lpFindFileData->nFileSizeLow=FindFileDataA.nFileSizeLow;
		}
		return retval;
	}
}

int MessageBoxT(HWND hWnd, const wchar_t* lpText, const wchar_t* lpCaption, UINT uType)
{
	if (usys()) {
		return MessageBoxW(hWnd, lpText, lpCaption, uType);
	} else {
		char *buf1 = (char*) malloc((wcslen(lpText) + 1)*sizeof(char));
		char *buf2 = (char*) malloc((wcslen(lpCaption) + 1)*sizeof(char));
		if (buf1 && buf2) {
			walcopy(buf1, lpText, wcslen(lpText));
			walcopy(buf2, lpCaption, wcslen(lpCaption));
			int res = MessageBox(hWnd, buf1, buf2, uType);
			free(buf1);
			free(buf2);
			return res;
		} else
			return 0;
	}
}

DWORD GetModuleFileNameT(HMODULE hModule, wchar_t* lpFileName, DWORD nSize)
{
	if (usys()) {
		return GetModuleFileNameW(hModule, lpFileName, nSize);
	} else {
		DWORD res = 0;
		char *buf = (char*) malloc((nSize + 1)*sizeof(char));
		if (buf) {
			res = GetModuleFileName(hModule, buf, nSize);
			if (res != 0)
				awlcopy(lpFileName, buf, nSize);
			free(buf);
		}
		return res;
	}
}

void DisplayLastErrorMsgT(const wchar_t* lpFunction, const wchar_t* lpFileName, UINT msgType)
{
    // retrieve the system error message for the last-error code
    LPVOID lpMsgBuf;
    LPVOID lpDisplayBuf;
    DWORD dw = GetLastError();

	if (usys()) {
	    FormatMessageW(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, dw, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (wchar_t*) &lpMsgBuf, 0, NULL );
	    // display the error message
	    lpDisplayBuf = (LPVOID)LocalAlloc(LMEM_ZEROINIT, (lstrlenW((const wchar_t*)lpMsgBuf) + lstrlenW((const wchar_t*)lpFunction) + 40)*sizeof(wchar_t));
		wsprintfW((wchar_t*)lpDisplayBuf, L"%s failed with error %d: %s", lpFunction, dw, lpMsgBuf);
		wchar_t wbuf[wdirtypemax+longnameprefixmax];
		if (MakeExtraLongNameW(wbuf, lpFileName, wdirtypemax-1+longnameprefixmax))
		    MessageBoxW(NULL, (const wchar_t*)lpDisplayBuf, (const wchar_t*)wbuf, msgType);
	} else {
	    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, dw, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (char*) &lpMsgBuf, 0, NULL );
	    // display the error message
		char *buf1 = (char*) malloc((wcslen(lpFunction) + 1)*sizeof(char));
		if (buf1) {
			walcopy(buf1, lpFunction, wcslen(lpFunction));
		    lpDisplayBuf = (LPVOID)LocalAlloc(LMEM_ZEROINIT, (lstrlen((const char*)lpMsgBuf) + lstrlen((const char*)buf1) + 40)*sizeof(char));
			wsprintf((char*)lpDisplayBuf, "%s failed with error %d: %s", buf1, dw, (char*)lpMsgBuf);
			char buf[_MAX_PATH];
		    MessageBox(NULL, (const char*)lpDisplayBuf, (const char*)wafilenamecopy(buf, lpFileName), msgType);
		    free(buf1);
		}
	}

    LocalFree(lpMsgBuf);
    LocalFree(lpDisplayBuf);
}

BOOL SetDlgItemTextT(HWND hDlg, int nIDDlgItem, const wchar_t* lpString)
{
	if (usys()) {
		return SetDlgItemTextW(hDlg, nIDDlgItem, lpString);
	}
	else {
		BOOL res = FALSE;
		char *buf = (char*) malloc((wcslen(lpString) + 1)*sizeof(char));
		if (buf) {
			walcopy(buf, lpString, wcslen(lpString));
			res = SetDlgItemText(hDlg, nIDDlgItem, buf);
			free(buf);
		}
		return res;
	}
}

UINT GetDlgItemTextT(HWND hDlg, int nIDDlgItem, wchar_t* lpString, int cchMax)
{
	if (usys()) {
		return GetDlgItemTextW(hDlg, nIDDlgItem, lpString, cchMax);
	}
	else {
		BOOL res = FALSE;
		char *buf = (char*) malloc((cchMax + 1)*sizeof(char));
		if (buf) {
			walcopy(buf, lpString, wcslen(lpString));
			res = GetDlgItemText(hDlg, nIDDlgItem, buf, cchMax);
			if (res)
				awlcopy(lpString, buf, cchMax);
			free(buf);
		}
		return res;
	}
}

DWORD GetPrivateProfileStringT(const wchar_t* lpAppName, const wchar_t* lpKeyName, const wchar_t* lpDefault, wchar_t* lpReturnedString, DWORD nSize, const wchar_t* lpFileName)
{
	if (usys()) {
		wchar_t wbuf[wdirtypemax+longnameprefixmax];
		if (MakeExtraLongNameW(wbuf, lpFileName, wdirtypemax-1+longnameprefixmax))
			return GetPrivateProfileStringW(lpAppName, lpKeyName, lpDefault, lpReturnedString, nSize, wbuf);
		else
			return 0;
	}
	else {
		char *buf1 = lpAppName ? (char*) malloc((wcslen(lpAppName) + 1)*sizeof(char)) : NULL;
		char *buf2 = lpKeyName ? (char*) malloc((wcslen(lpKeyName) + 1)*sizeof(char)) : NULL;
		char *buf3 = lpDefault ? (char*) malloc((wcslen(lpDefault) + 1)*sizeof(char)) : NULL;
		char *buf4 = (char*) malloc((nSize + 1)*sizeof(char));
		char buf5[_MAX_PATH];
		if ((lpAppName && buf1 && lpKeyName && buf2 && lpDefault && buf3 && buf4) || (buf4)) {
			if (lpAppName)
				walcopy(buf1, lpAppName, wcslen(lpAppName));
			if (lpKeyName)
				walcopy(buf2, lpKeyName, wcslen(lpKeyName));
			if (lpDefault)
				walcopy(buf3, lpDefault, wcslen(lpDefault));
			DWORD res = GetPrivateProfileString(buf1, buf2, buf3, buf4, nSize, wafilenamecopy(buf5, lpFileName));
			if (res > 0) {
				MultiByteToWideChar(CP_ACP, 0, buf4, res, lpReturnedString, nSize);
				lpReturnedString[res] = 0;
			}
			if (lpAppName)
				free(buf1);
			if (lpKeyName)
				free(buf2);
			if (lpDefault)
				free(buf3);
			free(buf4);
			return res;
		} else
			return 0;
	}
}
