// See http://www.gamedev.net/community/forums/topic.asp?topic_id=299121

#include <vector>
#include <assert.h>

template <class T, template <class U, class = std::allocator<U> > class storage = std::vector>
class unsorted_map
{
public:
	typedef std::pair<std::string, T> value_type;
	typedef typename storage<value_type>::const_iterator const_iterator;
	typedef typename storage<value_type>::iterator iterator;
	typedef typename storage<value_type>::const_reference const_reference;
	typedef typename storage<value_type>::reference reference;

	const_reference operator[](const std::string & key) const
	{
#ifdef _DEBUG //Make sure it exists if we're in debug mode
		const_iterator output = find_value(key);
		assert(output != _storage.end());
		return *output;
#else
		return *find_value(key);
#endif
	}

	reference operator[](const std::string & key)
	{
#ifdef _DEBUG //Make sure it exists if we're in debug mode
		iterator output = find_value(key);
		assert(output != _storage.end());
		return *output;
#else
		return *find_value(key);
#endif
	}

	const_iterator find(const std::string & key) const {return find_value(key);}
	iterator find(const std::string & key) {return find_value(key);}

	const_iterator begin() const {return _storage.begin();}
	iterator begin() {return _storage.begin();}

	const_iterator end() const {return _storage.end();}
	iterator end() {return _storage.end();}

	void clear() {_storage.clear();}
	bool empty() const {return _storage.empty();}

	std::string::size_type size() const {return _storage.size();}

	reference at(std::string::size_type _Pos) {return _storage.at(_Pos);}

	void push_back(const value_type & value)
	{
		assert(find_value(value.first) == _storage.end()); //Make sure it doesn't already exist!
		_storage.push_back();
	}
	void push_back(const std::string & key, const T & value)
	{
		assert(find_value(key) == _storage.end()); //Make sure it doesn't already exist!
		_storage.push_back(value_type(key, value));
	}
private:
	inline const_iterator find_value(const std::string & key) const
	{
		const_iterator start = _storage.begin(), end = _storage.end();
		for (; start != end; ++start) //Keep looping until end
		{
			if (start->first == key)
				break; //Found it, so break out of loop
		} //If we didn't find it then current == end
		return start;
	}
	inline iterator find_value(const std::string & key)
	{
		iterator start = _storage.begin(), end = _storage.end();
		for (; start != end; ++start) //Keep looping until end
		{
			if (start->first == key)
				break; //Found it, so break out of loop
		} //If we didn't find it then current == end
		return start;
	}

	storage<value_type> _storage;
};
